package com.ruoyi.file.mapper;

import com.ruoyi.file.domain.FilesAchievement;

import java.util.List;

/**
 * 成果表Mapper接口
 * 
 * @author cmc
 * @date 2022-09-06
 */
public interface FilesAchievementMapper 
{
    /**
     * 查询成果表
     * 
     * @param id 成果表主键
     * @return 成果表
     */
    public FilesAchievement selectFilesAchievementById(String id);

    /**
     * 查询成果表列表
     * 
     * @param filesAchievement 成果表
     * @return 成果表集合
     */
    public List<FilesAchievement> selectFilesAchievementList(FilesAchievement filesAchievement);

    /**
     * 查询项目成果列表
     *
     * @param filesAchievement 成果表
     * @return 成果表集合
     */
    public List<FilesAchievement> selectProjectAchievementList(FilesAchievement filesAchievement);

    /**
     * 新增成果表
     * 
     * @param filesAchievement 成果表
     * @return 结果
     */
    public int insertFilesAchievement(FilesAchievement filesAchievement);

    /**
     * 修改成果表
     * 
     * @param filesAchievement 成果表
     * @return 结果
     */
    public int updateFilesAchievement(FilesAchievement filesAchievement);

    /**
     * 删除成果表
     * 
     * @param id 成果表主键
     * @return 结果
     */
    public int deleteFilesAchievementById(String id);

    /**
     * 批量删除成果表
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteFilesAchievementByIds(String[] ids);

    /**
     * 根据项目id批量删除对应的成果列表
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteFilesAchievementByProjectIds(String[] ids);
}
